@@ -108,9 +108,8 @@ class AsyncFunction:
         if missing_args:
             logger.error(f"Missing required arguments: {missing_args}")
             missing_pos_args = [param for param in missing_args if param in self.posonly_params or param in self.pos_kw_params]
-            count = len(missing_pos_args)
-            plural = 's' if count != 1 else ''
-            raise TypeError(f"Async function '{self.node.name}' missing {count} required positional argument{plural}")
+            raise TypeError(f"Async function '{self.node.name}' missing {len(missing_pos_args)} required positional argument{'s' if len(missing_pos_args) != 1 else ''}")
+
         # Set up the interpreter with the new environment
         new_interp = self.interpreter.spawn_from_env(new_env_stack + [local_frame])
         
@@ -119,8 +118,8 @@ class AsyncFunction:
             logger.debug(f"Debug: local_frame before execution in {self.node.name}: {local_frame}")
             logger.debug("Beginning statement execution in {}".format(self.node.name))
             for stmt in self.node.body:
-                logger.debug(f"Executing statement: {stmt.__class__.__name__} in {self.node.name}")
-                last_value = await new_interp.visit(stmt, wrap_exceptions=True)
+                # Propagate StopAsyncIteration out of async function for user try/except
+                last_value = await new_interp.visit(stmt, wrap_exceptions=False)
             logger.debug(f"{self.node.name} completed all statements, last_value: {last_value}")
             if _return_locals:
                 local_vars = {k: v for k, v in local_frame.items() if not k.startswith('__')}
@@ -132,10 +131,18 @@ class AsyncFunction:
                 local_vars = {k: v for k, v in local_frame.items() if not k.startswith('__')}
                 return ret.value, local_vars
             return ret.value
-        except StopAsyncIteration as stop:
-            # Handle StopAsyncIteration to capture return value from async generators or manual StopAsyncIteration
+        except RuntimeError as e:
+            # Handle StopIteration raised from coroutine per PEP 479
+            if str(e) == "coroutine raised StopIteration":
+                orig = e.__cause__ if hasattr(e, '__cause__') else None
+                ret_val = getattr(orig, 'value', None)
+                logger.debug(f"{self.node.name} caught coroutine StopIteration with value: {ret_val}")
+                return ret_val
+            raise
+        except StopIteration as stop:
+            # Treat StopIteration as a return from async function
+            logger.debug(f"{self.node.name} caught StopIteration with value: {getattr(stop, 'value', None)}")
             ret_val = getattr(stop, 'value', None)
-            logger.debug(f"{self.node.name} caught StopAsyncIteration with value: {ret_val}")
             if _return_locals:
                 local_vars = {k: v for k, v in local_frame.items() if not k.startswith('__')}
                 return ret_val, local_vars
