@@ -11,34 +11,38 @@ async def visit_Try(interpreter, node: ast.Try, wrap_exceptions: bool = True) ->
         for stmt in node.body:
             # Execute body without wrapping to allow exceptions like StopAsyncIteration to propagate
             result = await interpreter.visit(stmt, wrap_exceptions=False)
-    except ReturnException:
+    except ReturnException as ret:
         interpreter.env_stack[0]['logger'].debug("Propagating ReturnException")
         # Propagate return after executing finalbody
-        raise
+        raise ret
     except Exception as e:
         interpreter.env_stack[0]['logger'].debug(f"Caught exception: {type(e).__name__}")
-        # Unwrap PEP 479 conversion of StopIteration to RuntimeError in async calls
-        if isinstance(e, RuntimeError) and isinstance(getattr(e, '__cause__', None), StopIteration):
-            interpreter.env_stack[0]['logger'].debug("Unwrapping PEP479 RuntimeError to StopIteration")
-            original_e = e.__cause__
-        else:
-            original_e = e
-        # Recursively unwrap the exception
-        while isinstance(original_e, WrappedException) and hasattr(original_e, "original_exception"):
-            original_e = original_e.original_exception
+        # Unwrap the exception
+        original_e = e
+        while isinstance(original_e, WrappedException):
+            original_e = getattr(original_e, 'original_exception', original_e)
+            if original_e is getattr(original_e, 'original_exception', None):
+                break
         interpreter.env_stack[0]['logger'].debug(f"Unwrapped exception: {type(original_e).__name__}")
         exception_raised = original_e
         # Track current exception to support bare raise
         interpreter.current_exception = original_e
+        interpreter.env_stack[0]['logger'].debug(f"Entering exception handler loop with {len(node.handlers)} handlers for exception type: {type(original_e).__name__}")
         for handler in node.handlers:
             exc_type = await interpreter._resolve_exception_type(handler.type) if handler.type else Exception
+            interpreter.env_stack[0]['logger'].debug(f"Debug: Handler type resolved to {exc_type.__name__ if exc_type else 'None'}, for handler at line {getattr(handler, 'lineno', 'unknown')} ")
+            interpreter.env_stack[0]['logger'].debug(f"Debug: Checking handler for exception type: {type(original_e).__name__} against {exc_type.__name__ if exc_type else 'None'}")
             interpreter.env_stack[0]['logger'].debug(f"Checking handler for exception type: {exc_type.__name__}")
             if isinstance(original_e, exc_type):
                 interpreter.env_stack[0]['logger'].debug("Handler matched")
                 if handler.name:
                     interpreter.set_variable(handler.name, original_e)
-                for stmt in handler.body:
-                    result = await interpreter.visit(stmt, wrap_exceptions=True)
+                try:
+                    for stmt in handler.body:
+                        result = await interpreter.visit(stmt, wrap_exceptions=True)
+                except ReturnException as ret:
+                    interpreter.env_stack[0]['logger'].debug("Propagating ReturnException from handler")
+                    raise ret
                 # Clear current_exception after handling
                 interpreter.current_exception = None
                 break
@@ -70,12 +74,15 @@ async def visit_Try(interpreter, node: ast.Try, wrap_exceptions: bool = True) ->
 
 
 async def visit_TryStar(interpreter, node: ast.TryStar, wrap_exceptions: bool = True) -> Any:
-    from .exceptions import WrappedException
+    from .exceptions import WrappedException, StopAsyncIterationWithValue
     result = None
     interpreter.env_stack[0]['logger'].debug("Entering visit_TryStar")
     try:
         for stmt in node.body:
             result = await interpreter.visit(stmt, wrap_exceptions=True)
+    except StopAsyncIterationWithValue as e:
+        interpreter.env_stack[0]['logger'].debug(f"Caught StopAsyncIterationWithValue in TryStar: {e.value}")
+        interpreter.env_stack[0]['__return__'] = e.value
     except Exception as e:
         interpreter.env_stack[0]['logger'].debug(f"Caught exception in TryStar: {type(e).__name__}")
         for handler in node.handlers:
